@@ -30,6 +30,8 @@ import (
 	"github.com/openshift/cloud-credential-operator/pkg/controller/internalcontroller"
 	"github.com/openshift/cloud-credential-operator/pkg/controller/utils"
 
+	configv1 "github.com/openshift/api/config/v1"
+
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -59,20 +61,23 @@ const (
 
 	cloudCredDeprovisionFailure = "CloudCredDeprovisionFailure"
 	cloudCredDeprovisionSuccess = "CloudCredDeprovisionSuccess"
+
+	credentialsRequestInfraMismatch = "InfrastructureMismatch"
 )
 
 // AddWithActuator creates a new CredentialsRequest Controller and adds it to the Manager with
 // default RBAC. The Manager will set fields on the Controller and Start it when
 // the Manager is Started.
-func AddWithActuator(mgr manager.Manager, actuator actuator.Actuator) error {
-	return add(mgr, newReconciler(mgr, actuator))
+func AddWithActuator(mgr manager.Manager, actuator actuator.Actuator, platType configv1.PlatformType) error {
+	return add(mgr, newReconciler(mgr, actuator, platType))
 }
 
 // newReconciler returns a new reconcile.Reconciler
-func newReconciler(mgr manager.Manager, actuator actuator.Actuator) reconcile.Reconciler {
+func newReconciler(mgr manager.Manager, actuator actuator.Actuator, platType configv1.PlatformType) reconcile.Reconciler {
 	return &ReconcileCredentialsRequest{
-		Client:   mgr.GetClient(),
-		Actuator: actuator,
+		Client:       mgr.GetClient(),
+		Actuator:     actuator,
+		platformType: platType,
 	}
 }
 
@@ -83,7 +88,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	name := "credentialsrequest-controller"
+	name := "credentialsrequest_controller"
 
 	// Custom rateLimiter that sets minimum backoff to 2 seconds
 	rateLimiter := workqueue.NewMaxOfRateLimiter(
@@ -98,7 +103,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		Config:                  mgr.GetConfig(),
 		Scheme:                  mgr.GetScheme(),
 		Client:                  mgr.GetClient(),
-		Recorder:                mgr.GetRecorder(name),
+		Recorder:                mgr.GetEventRecorderFor(name),
 		Queue:                   workqueue.NewNamedRateLimitingQueue(rateLimiter, name),
 		MaxConcurrentReconciles: 1,
 		Name:                    name,
@@ -187,7 +192,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 			newNamespace := a.Meta.GetName()
 			log.WithField("namespace", newNamespace).Debug("checking for credentials requests targeting namespace")
 			crs := &minterv1.CredentialsRequestList{}
-			mgr.GetClient().List(context.TODO(), &client.ListOptions{}, crs)
+			mgr.GetClient().List(context.TODO(), crs)
 			requests := []reconcile.Request{}
 			for _, cr := range crs.Items {
 				if !cr.Status.Provisioned && cr.Spec.SecretRef.Namespace == newNamespace {
@@ -238,7 +243,8 @@ var _ reconcile.Reconciler = &ReconcileCredentialsRequest{}
 // ReconcileCredentialsRequest reconciles a CredentialsRequest object
 type ReconcileCredentialsRequest struct {
 	client.Client
-	Actuator actuator.Actuator
+	Actuator     actuator.Actuator
+	platformType configv1.PlatformType
 }
 
 // Reconcile reads that state of the cluster for a CredentialsRequest object and
@@ -249,7 +255,7 @@ type ReconcileCredentialsRequest struct {
 // +kubebuilder:rbac:groups=core,resources=secrets;configmaps;events,verbs=get;list;watch;create;update;patch;delete
 // +kubebuilder:rbac:groups=core,resources=namespaces,verbs=get;list;watch
 // +kubebuilder:rbac:groups=config.openshift.io,resources=clusterversions,verbs=get;list;watch
-// +kubebuilder:rbac:groups=config.openshift.io,resources=infrastructures,verbs=get;list;watch
+// +kubebuilder:rbac:groups=config.openshift.io,resources=infrastructures;dnses,verbs=get;list;watch
 // +kubebuilder:rbac:groups=config.openshift.io,resources=clusteroperators;clusteroperators/status,verbs=create;get;update;list;watch
 func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reconcile.Result, error) {
 	logger := log.WithFields(log.Fields{
@@ -284,6 +290,22 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 	origCR := cr
 	cr = cr.DeepCopy()
 
+	// Ignore CR if it's for a different cloud/infra
+	infraMatch, err := crInfraMatches(cr, r.platformType)
+	if err != nil {
+		logger.WithError(err).Error("failed to determine cloud platform type")
+		return reconcile.Result{}, err
+	}
+	if !infraMatch {
+		logger.Debug("ignoring cr as it is for a different cloud")
+		setIgnoredCondition(cr, r.platformType)
+		err := r.updateStatus(origCR, cr, logger)
+		if err != nil {
+			logger.WithError(err).Error("failed to update conditions")
+		}
+		return reconcile.Result{}, err
+	}
+
 	// Handle deletion and the deprovision finalizer:
 	if cr.DeletionTimestamp != nil {
 		if HasFinalizer(cr, minterv1.FinalizerDeprovision) {
@@ -398,6 +420,8 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 	}
 	if syncErr != nil {
 		logger.Errorf("error syncing credentials: %v", syncErr)
+		// TODO: set condition if previously satisfied credrequest can now
+		// not be satisfied (but keeping provisioned==True).
 		cr.Status.Provisioned = false
 
 		switch t := syncErr.(type) {
@@ -532,6 +556,24 @@ func setCredentialsDeprovisionFailureCondition(cr *minterv1.CredentialsRequest,
 		status, reason, msg, updateCheck)
 }
 
+func setIgnoredCondition(cr *minterv1.CredentialsRequest, clusterPlatform configv1.PlatformType) {
+	// Only supporting the ability to set the condition
+	msg := fmt.Sprintf("CredentialsRequest is not for platform %s", clusterPlatform)
+	reason := credentialsRequestInfraMismatch
+	updateCheck := utils.UpdateConditionIfReasonOrMessageChange
+	status := corev1.ConditionTrue
+
+	cr.Status.Conditions = utils.SetCredentialsRequestCondition(cr.Status.Conditions, minterv1.Ignored,
+		status, reason, msg, updateCheck)
+
+	// Also clear any other conditions since we are ignoring this cred request,
+	// and we don't want to be in a degraded state b/c of cred requests that we're ignoring.
+	for _, cond := range failureConditionTypes {
+		cr.Status.Conditions = utils.SetCredentialsRequestCondition(cr.Status.Conditions, cond,
+			corev1.ConditionFalse, reason, msg, updateCheck)
+	}
+}
+
 func (r *ReconcileCredentialsRequest) updateStatus(origCR, newCR *minterv1.CredentialsRequest, logger log.FieldLogger) error {
 	logger.Debug("updating credentials request status")
 
@@ -583,3 +625,23 @@ func DeleteFinalizer(object metav1.Object, finalizer string) {
 	finalizers.Delete(finalizer)
 	object.SetFinalizers(finalizers.List())
 }
+
+func crInfraMatches(cr *minterv1.CredentialsRequest, clusterCloudPlatform configv1.PlatformType) (bool, error) {
+	cloudType, err := utils.GetCredentialsRequestCloudType(cr.Spec.ProviderSpec)
+	if err != nil {
+		return true, fmt.Errorf("error determining cloud type for CredentialsRequest: %v", err)
+	}
+
+	switch clusterCloudPlatform {
+	case configv1.AWSPlatformType:
+		return cloudType == reflect.TypeOf(minterv1.AWSProviderSpec{}).Name(), nil
+	case configv1.AzurePlatformType:
+		return cloudType == reflect.TypeOf(minterv1.AzureProviderSpec{}).Name(), nil
+	case configv1.GCPPlatformType:
+		return cloudType == reflect.TypeOf(minterv1.GCPProviderSpec{}).Name(), nil
+	case configv1.OpenStackPlatformType:
+		return cloudType == reflect.TypeOf(minterv1.OpenStackProviderSpec{}).Name(), nil
+	default:
+		return false, fmt.Errorf("unsupported platorm type: %v", clusterCloudPlatform)
+	}
+}
